/*********************************************\
*   BeApple2 : Apple ][+ emulator for BeOS    *
*                                             *
*        speaker.cpp : Sound                  *
\*********************************************/

#include "config.h"
#include "cpu.h"
#include "tape.h"
#include <MediaKit.h>
#include <stdio.h>

const uint SURECH=8;	// Suréchantillonnage (puissance de 2)
const uint FILTRE=12; // Recouvrement filtrage
const float FREQSON=44100.0; // Fréquence échantillonnage

float SpkTps=0;	// Temporisation relative speaker, en ms

int SpkFilter=1;				// Sélection filtre
uint8 SpkSon[8*1024];			// Buffer son
uint8 SpkTur[2*SURECH*1024];	// Buffer suréchantillonnage
uint8 SpkBuf[8];				// Remplissure buffers
int SpkNBC=0;					// 0-7 : Buffer courant écriture
	// Supposons que la précision relative des 2 quartz est supérieure à 0.2% ...
float SpkPer=1024.0*1000.0/FREQSON;			// Période bloc de 1024 échs. en ms v.i. = 23.220
float SpkPerNom=1024.0*1000.0/FREQSON;			// Période bloc de 1024 échs. en ms v.i. = 23.220

float SpkPerMin=1024*1000.0/FREQSON*0.998;
float SpkPerMax=1024*1000.0/FREQSON*1.002;

float Filter[20];	// Filter op.

BSoundPlayer *Ophi; // Ophicléide

media_raw_audio_format MRAF;

bool SpkTog=false;	// Toggle speaker.
int	 Nob=0;			// Buffer courant lecture

bool Musica=false; // ¿ Vamos a oír música ?
//------------------------------------------
// Appel IO Speaker
void  Speak()
{
 if (!Musica) return;
 /*printf ("SPK:Beep\n");*/
 float Tps=SpkTps+(float)FasCycles/(float)Speed; // Instant relatif, µs
 int64 NE=(int64)((float)Tps*(float)(1024.0*SURECH)/(float)SpkPer+0.5);
 int X=NE&(1024*2*SURECH-1);	// Pos. tableau
 SpkTur[X]++;				// Surmodulation ???
 return;
}

//------------------------------------------
// Updates sound buffer
void SpeakLoop(int64 SpkCycles)
{
 if (!Musica) return;
 SpkTps+=(float)SpkCycles/(float)Speed;
 if (SpkTps<SpkPer) return;
 SpkTps-=SpkPer;

 int K=0;
 for (int I=0;I<1024;I++) {
	int S=0;
	for (uint J=0;J<SURECH;J++,K++) {
		if (SpkTur[K]&1) SpkTog=!SpkTog;
		S+=SpkTog?1:0;
		}
	SpkSon[I+SpkNBC*1024]=S;
	}

 for (uint I=0;I<1024*SURECH;I++) { SpkTur[I]=SpkTur[I+1024*SURECH]; SpkTur[I+1024*SURECH]=0; }

 //for (int I=0;I<1024;I++) SpkSon[I+SpkNBC*1024]=((I/32)&1); // Purity
 SpkBuf[SpkNBC]=1;
 SpkNBC=(SpkNBC+1)&7;

 int NBP=	SpkBuf[ 0]+SpkBuf[ 1]+SpkBuf[ 2]+SpkBuf[ 3]+SpkBuf[ 4]+SpkBuf[ 5]+SpkBuf[ 6]+SpkBuf[ 7];
 if (NBP<3) SpkPer=(SpkPer>SpkPerMin)?(SpkPer-0.001):SpkPer;
 if (NBP==3) SpkPer=SpkPerNom;
 if (NBP==4) SpkPer=SpkPerNom;
 if (NBP>4) SpkPer=(SpkPer<SpkPerMax)?(SpkPer+0.001):SpkPer;
/* if (NBP==0) printf ("NBP=0!!!\n");
 if (NBP==7) printf ("NBP=7!!!\n");
*/
 }

//##############################################################
//##############################################################

//------------------------------------------
// Callback
void BufferProc(void *Cookie, void *Buffer,size_t Size,const media_raw_audio_format &Format)
{
 size_t I;
 float *Buf = (float *)Buffer;
 float T;
 float But[1100];
 Size/=sizeof (float);
 if (Format.format != media_raw_audio_format::B_AUDIO_FLOAT) return;

 for (I=0;I<1024;I++) {
	T=SpkSon[I+Nob*1024]-(float)(SURECH)/2.0;
	But[I+FILTRE] =T/(float)(SURECH)/2;
	}
 switch(SpkFilter) {
 	case 0: { // No filter
 		for (I=0;I<1024;I++) Buf[I]=But[I+FILTRE];
 		break;
 		}
 	case 1: { // 4 taps 'averaging' FIR filter
		for (I=0;I<FILTRE;I++) But[I]=Filter[I];
		for (I=0;I<FILTRE;I++) Filter[I]=But[I+1024];
		float F;
		for (I=0;I<1024;I++) {
	 		F=But[I]+But[I+1]+But[I+2]+But[I+3];
	 		Buf[I]=F/4;
			}
		break;
		}
	case 2: { // 10 taps Chebyshev FIR filter
		for (I=0;I<FILTRE;I++) But[I]=Filter[I];
		for (I=0;I<FILTRE;I++) Filter[I]=But[I+1024];
		for (I=0;I<1024;I++) {
			Buf[I]= -0.030*(But[I  ]+But[I+9])
			//	+0.000*(But[I+1]+But[I+8])
				+0.087*(But[I+2]+But[I+7])
				+0.197*(But[I+3]+But[I+6])
				+0.271*(But[I+4]+But[I+5]);
			}
		break;
		}
	case 3: { // 12 taps Chebyshev FIR filter
		for (I=0;I<FILTRE;I++) But[I]=Filter[I];
		for (I=0;I<FILTRE;I++) Filter[I]=But[I+1024];
		for (I=0;I<1024;I++) {
			Buf[I]= -0.045*(But[I  ]+But[I+11])
				//	+0.000*(But[I+1]+But[I+10])
					+0.036*(But[I+2]+But[I+9])
					+0.112*(But[I+3]+But[I+8])
					+0.188*(But[I+4]+But[I+7])
					+0.234*(But[I+5]+But[I+6]);
			}
		break;
		}
	}
 if (TapeEcr) WriteTape(Buf);

 if (SpkBuf[Nob]) {
 	SpkBuf[Nob]=0;
 	} else { for (I=0;I<1024;I++) Buf[I]=0; Nob=(Nob+1)&7;/* printf ("Déverouillé\n");*/ }
Nob=(Nob+1)&7;
}
//------------------------------------------

void InitSon()
{
 Musica=false;
 MRAF.format=media_raw_audio_format::B_AUDIO_FLOAT;
 MRAF.frame_rate=FREQSON;
 MRAF.channel_count=1;
 MRAF.byte_order=B_MEDIA_LITTLE_ENDIAN;
 MRAF.buffer_size=4096;
 Ophi=new BSoundPlayer(&MRAF,(const char *)"BeApple][",BufferProc,NULL);
 status_t stat=Ophi->InitCheck();
 if (stat!=B_OK) {
 	delete Ophi;
 	BAlert *baAlert = new BAlert("Speaker", "I can't play sound !","Too bad",(char *)NULL,(char *)NULL
 								,B_WIDTH_AS_USUAL, B_OFFSET_SPACING, B_WARNING_ALERT);
	baAlert->SetShortcut(0, B_ESCAPE);
	baAlert->Go();
	return;
	}
 Ophi->Start();
 Ophi->SetHasData(true);
 Musica=true;
}

//------------------------------------------
void ClearSon()
{
 if (!Musica) return;
 
 Ophi->Stop();
 delete Ophi;
}

